package se.gawell.fakeriak.index;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.codehaus.jackson.map.ObjectMapper;

public class InMemoryIndexes implements Indexes {

	private ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>>> indexBucketMap = new ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>>>();

	@Override
	public String getKeysAsJsonString(String bucketName, String indexName, String indexValueA, String indexValueB) {
		List<String> bucketIndexValueList = new LinkedList<String>();  
		
		if (indexValueB == null) {
			indexValueB = indexValueA;
		}
		
		for (String indexValue: getBucketIndex(bucketName, indexName).keySet()) {
			if (indexValueA.compareTo(indexValue) <= 0
					&& indexValueB.compareTo(indexValue) >= 0) {
				ConcurrentLinkedQueue<String> valuesForKey = getBucketIndexValueSet(bucketName, indexName, indexValue);
				bucketIndexValueList.addAll(valuesForKey);
			}
		}
		
		Collections.sort(bucketIndexValueList);
		
		ObjectMapper objectMapper = new ObjectMapper();
		try {
			return objectMapper.writeValueAsString(new KeyList(bucketIndexValueList));
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public void put(String bucketName, String indexName, String indexValue, String key) {
		ConcurrentLinkedQueue<String> bucketIndexValueSet = getBucketIndexValueSet(bucketName, indexName, indexValue);
		bucketIndexValueSet.add(key);
	}

	private ConcurrentLinkedQueue<String> getBucketIndexValueSet(String bucketName, String indexName, String indexValue) {
		final ConcurrentHashMap<String, ConcurrentLinkedQueue<String>> bucketIndex = getBucketIndex(bucketName, indexName);
		ConcurrentLinkedQueue<String> indexSet = bucketIndex.get(indexValue);
		if (indexSet == null) {
			indexSet = new ConcurrentLinkedQueue<String>();
			bucketIndex.put(indexValue, indexSet);
		}
		return indexSet;
	}

	private ConcurrentHashMap<String, ConcurrentLinkedQueue<String>> getBucketIndex(String bucketName, String indexName) {
		final ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>> bucket = getBucket(bucketName);
		ConcurrentHashMap<String, ConcurrentLinkedQueue<String>> index = bucket.get(indexName);
		if (index == null) {
			index = new ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>();
			bucket.put(indexName, index);
		}
		return index;
	}

	private ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>> getBucket(String bucketName) {
		ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>> indexBucket = indexBucketMap.get(bucketName);
		if (indexBucket == null) {
			indexBucket = new ConcurrentHashMap<String, ConcurrentHashMap<String, ConcurrentLinkedQueue<String>>>();
			indexBucketMap.put(bucketName, indexBucket);
		}
		return indexBucket;
	}

}
